export declare const getTemplateDirectory: () => string;
export declare const getTemplateFilePath: (template: string) => string;
export declare const getBuiltInTemplates: () => {
    css: {
        path: string;
    };
    html: {
        path: string;
    };
    json: {
        path: string;
    };
    scss: {
        path: string;
    };
    styl: {
        path: string;
    };
};
